@@ -6,9 +6,10 @@
 #  include <config.h>
 #endif
 
- #include <pwd.h>
- #include <time.h>
+#include <pwd.h>
+#include <time.h>
 #include <gtk/gtk.h>
+#include  <glib/gi18n.h>
 #include <glib/gprintf.h>
 #include <libgnomeui/libgnomeui.h>
 #include <libgnomeui/gnome-ui-init.h>
@@ -16,20 +17,18 @@
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <gconf/gconf-client.h>
+#include <glibtop.h>
  
 #include "baobab.h"
 #include "tv.h"
-#include "interface.h"
-#include "support.h"
-#include "utilities.h"
 #include "bbthread.h"
 #include "callbacks.h"
 #include "props.h"
 
-	static GnomeVFSMonitorHandle * handle_mtab;
-	static GnomeVFSMonitorHandle * handle_home;
+static GnomeVFSMonitorHandle * handle_mtab;
+static GnomeVFSMonitorHandle * handle_home;
 
-static 	void prefill_model (gchar* );
+static 	void prefill_model (struct chan_data* );
 
 static void check_UTF(GString *);
 static void  push_iter_in_stack (GtkTreeIter * );
@@ -42,7 +41,6 @@ static GQueue * iterstack=NULL;
 static void prepare_firstcol(GString*, struct BaobabSearchRet* );
 static gchar* get_owner(uid_t );
 static gchar* get_last_mod(time_t );
-static const gchar* get_file_filter (gchar* );
 gboolean filter_adv_search(struct BaobabSearchRet *);
 static void initialize_search_variables(void);
 static void baobab_init(void);
@@ -54,25 +52,44 @@ static void baobab_close(void);
 void
 start_proc_on_dir (gchar *dir)
 {
-	
+	GdkCursor *cursor = NULL;
+	GtkWidget *ck_allocated;
+
 	switch_view(VIEW_TREE);
-	if (!check_dir(dir)) { return;}
+
+	if (!baobab_check_dir(dir))
+		return;
+
 	g_string_assign(baobab.last_scan_command,dir);
 	baobab.STOP_SCANNING=FALSE;
 	change_icon("busy.gif");
 	check_menu_sens(TRUE);
 	gtk_tree_store_clear(baobab.model);
-	currentdepth=-1;			//flag
+	currentdepth=-1;			/* flag */
 	iterstack = g_queue_new ();
+
+	/* check if the file system is local or remote */
+	baobab.is_local = bb_scan_is_local(dir);
+	ck_allocated = glade_xml_get_widget(baobab.main_xml,"ck_allocated");
+	if (!baobab.is_local) {
+		gtk_toggle_button_set_active((GtkToggleButton*)ck_allocated,FALSE);
+		gtk_widget_set_sensitive (ck_allocated, FALSE);
+		baobab.show_allocated = FALSE;			  
+	}
+	else {
+		gtk_widget_set_sensitive (ck_allocated,TRUE);	
+	}
+	
 		
 	getDir(dir);
 
-	//change the cursor
-	GdkCursor *cursor = NULL;
+	/* change the cursor */
 	if (baobab.window->window) {
  		cursor = gdk_cursor_new(GDK_WATCH);
 		gdk_window_set_cursor (baobab.window->window, cursor);
 			}
+			
+	/* set statusbar, percentage and allocated/normal size */
 	set_statusbar( _("Calculating percentage bars..."));
 	gtk_tree_model_foreach((GtkTreeModel*)baobab.model,show_bars,NULL);
 	change_icon("done.png");
@@ -89,30 +106,46 @@ start_proc_on_dir (gchar *dir)
 	baobab.STOP_SCANNING=TRUE;
 	g_queue_free (iterstack);
 	baobab.CONTENTS_CHANGED_DELAYED=FALSE;
+
 }
 
 
 void
 start_search (gchar * searchname, gchar * dir)
 {
+	GdkCursor *cursor = NULL;
+
 	switch_view(VIEW_SEARCH);
 	initialize_search_variables();
 	baobab.STOP_SCANNING=FALSE;
 	change_icon("busy.gif");
 	check_menu_sens(TRUE);
-	while (gtk_events_pending()) { gtk_main_iteration(); }
+
+	/* change the cursor */
+	if (baobab.window->window) {
+ 		cursor = gdk_cursor_new(GDK_WATCH);
+		gdk_window_set_cursor (baobab.window->window, cursor);
+			}
+			
 	gtk_list_store_clear(baobab.model_search);
 
 	if (!dir) {
-		searchDir("/",searchname);		
+		searchDir("file:///",searchname);		
 	}
 	else {
 		searchDir(dir,searchname);
 	}
 	
+	/* cursor clean up */
+	if (baobab.window->window) {
+		gdk_window_set_cursor(baobab.window->window,NULL);
+		if (cursor) gdk_cursor_unref(cursor);
+		}
+
 	change_icon("done.png");
 	check_menu_sens(FALSE);
 	set_statusbar(_("Ready"));
+
 	set_label_search(baobab.number_found_files,baobab.size_found_files);
 	if (get_NB_page()==VIEW_SEARCH) show_label(VIEW_SEARCH);
 	gtk_tree_view_columns_autosize((GtkTreeView *)baobab.tree_search); 
@@ -129,12 +162,13 @@ fill_search_model(struct BaobabSearchRet *bbret)
 	GtkTreeIter iter;
 	GString * testo1;
 	GdkPixbuf * picon;
+	char * gicon;
 	
 	if (!filter_adv_search(bbret)) {
 			return;
 	}
 	
-	char* gicon=gnome_icon_lookup_sync(gtk_icon_theme_get_default(),NULL,bbret->fullpath,
+	gicon=gnome_icon_lookup_sync(gtk_icon_theme_get_default(),NULL,bbret->fullpath,
 								NULL,GNOME_ICON_LOOKUP_FLAGS_NONE,NULL);
 	picon= gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),gicon,36,GTK_ICON_LOOKUP_USE_BUILTIN,NULL);
 	
@@ -146,9 +180,9 @@ fill_search_model(struct BaobabSearchRet *bbret)
 	gtk_list_store_set(baobab.model_search, &iter,
 					COL0_ICON,picon,
 					COL1_STRING,testo1->str,
-					//COL2_STRING,testo2->str,
+					/* COL2_STRING,testo2->str, */
 					COL_FULLPATH,bbret->fullpath,
-					COL_FILETYPE,get_file_filter(bbret->fullpath),
+					COL_FILETYPE,bbret->mime_type,
 					COL_SIZE, bbret->size,
 					COL_LASTACCESS,bbret->lastacc,
 					COL_OWNER, bbret->owner,    
@@ -253,7 +287,7 @@ prepare_firstcol(GString* text,struct BaobabSearchRet* bbret)
 			size,
 			alloclabel->str,
 			alloc_size,
-			get_file_filter(bbret->fullpath));
+			gnome_vfs_mime_get_description(bbret->mime_type));
 	g_string_free(modlabel,TRUE);
 	g_string_free(ownlabel,TRUE);
 	g_string_free(fullabel,TRUE);
@@ -298,28 +332,15 @@ get_last_mod(time_t timeacc)
 	return g_string_free(time,FALSE);
 }
 
-const gchar*
-get_file_filter (gchar* file)
-{
-	const char* mime_type;
-	mime_type= gnome_vfs_get_file_mime_type(file, NULL, FALSE);
-	if (mime_type == NULL) {
-		return gnome_vfs_mime_get_description (GNOME_VFS_MIME_TYPE_UNKNOWN);
-	}
-	else {
-	return(gnome_vfs_mime_get_description(mime_type));
-	}
-	
-}
 /* pre-fills model during scanning
 */
 void
-prefill_model (gchar* buf)
+prefill_model (struct chan_data * data)
 {
 	GString * cdir;
+	char * basename;
 	GtkTreeIter iter, iterparent;
-	struct chan_data *data;
-	data = (struct chan_data *) buf;
+	GtkTreePath *path;
 
 	cdir=g_string_new ("");
 	
@@ -329,9 +350,9 @@ prefill_model (gchar* buf)
 	}
 	else if(data->depth==1) {
 		gtk_tree_store_append(baobab.model,&iter,&firstiter);
-		gtk_tree_view_expand_row (
-				(GtkTreeView*)baobab.tree_view,gtk_tree_model_get_path(
-							GTK_TREE_MODEL(baobab.model),&firstiter),FALSE);
+		path = gtk_tree_model_get_path(GTK_TREE_MODEL(baobab.model),&firstiter);
+		gtk_tree_view_expand_row ((GtkTreeView*)baobab.tree_view,path,FALSE);
+		gtk_tree_path_free(path);
 	}
 	else if(data->depth > currentdepth) {
 		gtk_tree_store_append(baobab.model,&iter,&currentiter);	
@@ -354,17 +375,21 @@ prefill_model (gchar* buf)
 	push_iter_in_stack(&iter);
 	currentiter=iter;
 
-	g_string_assign(cdir, g_filename_display_basename(data->dir));
+	basename = g_filename_display_basename(data->dir);
+	g_string_assign(cdir, basename);
+	g_free(basename);
 	
-	//check UTF-8 and locale
+	/* check UTF-8 and locale */
 	check_UTF(cdir);
+
 	g_string_append(cdir," &lt;== ");
 	g_string_append(cdir,_("scanning..."));
 	gtk_tree_store_set(baobab.model,&iter,COL_DIR_NAME,cdir->str,COL_H_FULLPATH, "",
 							COL_H_ELEMENTS,-1,-1);
 	while (gtk_events_pending()) { gtk_main_iteration(); }
 	g_string_free(cdir,TRUE);
-			
+
+		
 }
 
 /* set filesystem first row
@@ -381,7 +406,7 @@ first_row(void)
 	
 	gtk_tree_store_set(baobab.model,&firstiter,
 			COL_DIR_NAME, _("<i>Total filesystem usage:</i>"),	
-			COL_H_FULLPATH, "/", 
+			COL_H_FULLPATH, "", 
 			COL_BAR, set_bar((g_fs.used*100)/g_fs.total),
 			COL_H_PERC, (gfloat)((g_fs.used*100)/g_fs.total),
 			COL_DIR_SIZE, size,	
@@ -399,14 +424,14 @@ first_row(void)
 /* fills model during scanning
 *  model:
 *
-*		0, $dir,	                						1, $fullpath,
-*		2, set_bar($perc),									3, $perc,
-*		4, calc($size),	        							5, $size,
-*		6, sprintf("% 4s",$elements)." "._('elements'),		7, $elements,
-*		8, text showing hardlink size,						9, $HLsize
+*		0, $dir,					   1, $fullpath,
+*		2, set_bar($perc),				   3, $perc,
+*		4, calc($size),	        			   5, $size,
+*		6, sprintf("% 4s",$elements)." "._('elements'),	   7, $elements,
+*		8, text showing hardlink size,			   9, $HLsize
 */
 void
-fill_model (gchar* buf)
+fill_model (struct chan_data * data)
 {
 	GtkTreeIter iter;
 	GString *hardlinks;
@@ -414,15 +439,16 @@ fill_model (gchar* buf)
 	GString *elementi;
 	char *size;
 	char *alloc_size;
+	char * basename_cstr;
 	
-	struct chan_data *data;
-	data = (struct chan_data *) buf;
 	if (data->elements ==-1) {
-			prefill_model(buf);
+			prefill_model(data);
 			return;
 			}
 	basename= g_string_new ("");
-	g_string_assign(basename, g_filename_display_basename(data->dir));
+	basename_cstr = g_filename_display_basename(data->dir);
+	g_string_assign(basename, basename_cstr);
+	g_free(basename_cstr);
 	
 	check_UTF(basename);
 	g_string_prepend(basename,"<b>");
@@ -442,19 +468,21 @@ fill_model (gchar* buf)
 	}		
 
 	elementi=g_string_new ("");
-	g_string_printf(elementi,"% 5d %s",data->elements,
-		(data->elements ==1 ?_("object") : _("objects")));
+	g_string_printf(elementi,
+			ngettext("% 5d object", "% 5d objects", data->elements),
+			data->elements);
 	
 	size = gnome_vfs_format_file_size_for_display(data->size);
+
 	alloc_size = gnome_vfs_format_file_size_for_display(data->alloc_size);
 	
 	gtk_tree_store_set(baobab.model,&iter,
-			COL_DIR_NAME, basename->str,			COL_H_FULLPATH, data->dir,
+			COL_DIR_NAME, basename->str, COL_H_FULLPATH, data->dir,
 			COL_H_PERC, 0.0,
 			COL_DIR_SIZE, baobab.show_allocated ? alloc_size : size,			
 			COL_H_SIZE, data->size,
-			COL_ELEMENTS, elementi->str,			COL_H_ELEMENTS, data->elements,
-			COL_HARDLINK, hardlinks->str,			COL_H_HARDLINK, data->tempHLsize,
+			COL_ELEMENTS, elementi->str, COL_H_ELEMENTS, data->elements,
+			COL_HARDLINK, hardlinks->str, COL_H_HARDLINK, data->tempHLsize,
 			COL_H_ALLOCSIZE,data->alloc_size,
 			-1);
 	
@@ -464,12 +492,11 @@ fill_model (gchar* buf)
 	g_string_free(elementi,TRUE);
 	g_free(size);
 	g_free(alloc_size);
-
-			
+		
 }
 
 
-///////////////////////////////
+
 void 
 push_iter_in_stack (GtkTreeIter *iter)
 {
@@ -480,7 +507,7 @@ push_iter_in_stack (GtkTreeIter *iter)
 	g_queue_push_head (iterstack,GINT_TO_POINTER(iter->stamp));
 }
 
-///////////////////////////////
+
 GtkTreeIter
 pop_iter_from_stack (void)
 {
@@ -493,45 +520,88 @@ pop_iter_from_stack (void)
 	return iter;
 }
 
-///////////////////////////////////
+
 void
 check_UTF(GString * name)
 {
-	g_string_assign(name,g_filename_to_utf8(name->str,-1,NULL,NULL,NULL));
-	if (!name->str) {
-			g_string_assign(name,g_locale_to_utf8(name->str,-1,NULL,NULL,NULL));
-			if (!name->str)		{
-				g_string_assign(name,"<i>");
-				g_string_append(name, _("Invalid UTF-8 characters"));
-				g_string_append(name,"</i>");
-			}
+	char * str;
+	char * escaped_str;
+
+	str = g_filename_to_utf8(name->str, -1, NULL, NULL, NULL);
+
+	if (!str) {
+		str = g_locale_to_utf8(name->str, -1, NULL, NULL, NULL);
+
+		if (!str) {
+			str = g_strjoin("<i>",
+					_("Invalid UTF-8 characters"),
+					"</i>",
+					NULL);
 		}
-	
-	g_string_assign(name,g_markup_escape_text (name->str,name->len));
+	}
+
+	g_assert(str);
+
+	escaped_str = g_markup_escape_text(str, strlen(str));
+
+	g_string_assign(name, escaped_str);
+
+	g_free(str);
+	g_free(escaped_str);
 }
 
 
+gint 
+list_find  (gconstpointer a, gconstpointer b)
+{
+	gchar * str_a, * str_b;
+	gint ret;
+	
+	str_a = gnome_vfs_format_uri_for_display(a);
+	str_b = gnome_vfs_format_uri_for_display(b);
+	
+	ret = (strcmp(str_a,str_b));
+
+	g_free(str_a);
+	g_free(str_b);
+	return ret;
+	
+}
 
 
-
-void
+static void
 baobab_init(void)
 {
+	GConfClient * gc_client;
+	GnomeVFSResult result;
+	GnomeVFSVolumeMonitor* volmonitor;
+	
+	/* Load Glade */
+	baobab.glade_path = g_build_path(G_DIR_SEPARATOR_S,
+			    GLADE_DIR,
+			    "baobab.glade",
+			    NULL);
+	baobab.main_xml = glade_xml_new(baobab.glade_path,"baobab_window",NULL);
+	glade_xml_signal_autoconnect (baobab.main_xml);
+	
+
+	/* Misc */
 	baobab.label_scan = NULL;
 	baobab.label_search = NULL;
 	baobab.last_scan_command = g_string_new("/");
 	baobab.CONTENTS_CHANGED_DELAYED=FALSE;
 	baobab.STOP_SCANNING= TRUE;
 	baobab.show_allocated=TRUE;
+	baobab.is_local=TRUE;
 
-	//GConf
-	GConfClient * gc_client =gconf_client_get_default();
+	/* GConf */
+	gc_client = gconf_client_get_default();
 	gconf_client_add_dir(gc_client,BAOBAB_KEY, GCONF_CLIENT_PRELOAD_NONE,NULL);
 	gconf_client_notify_add(gc_client,PROPS_SCAN_KEY,props_notify,NULL,NULL,NULL);
 	baobab.bbExcludedDirs = gconf_client_get_list(gc_client,PROPS_SCAN_KEY,GCONF_VALUE_STRING,NULL);
-    baobab.bbEnableHomeMonitor = gconf_client_get_bool(gc_client,PROPS_ENABLE_HOME_MONITOR,NULL);
+    	baobab.bbEnableHomeMonitor = gconf_client_get_bool(gc_client,PROPS_ENABLE_HOME_MONITOR,NULL);
 	
-	//initialize bbSearchOpt variables
+	/* initialize bbSearchOpt variables */
 	bbSearchOpt.filename = g_string_new("");
 	bbSearchOpt.dir = g_string_new(g_get_home_dir());
 	bbSearchOpt.mod_date=NONE;
@@ -540,16 +610,16 @@ baobab_init(void)
 	bbSearchOpt.search_whole=FALSE;
 	bbSearchOpt.dont_recurse_dir=FALSE;
 	
-	//start VFS monitoring
+	/* start VFS monitoring */
 	handle_home=NULL;	
 
-	GnomeVFSVolumeMonitor* volmonitor=gnome_vfs_get_volume_monitor();
+	volmonitor = gnome_vfs_get_volume_monitor();
 	 g_signal_connect (volmonitor, "volume_mounted",
 		    G_CALLBACK (volume_changed), NULL);
   	g_signal_connect (volmonitor, "volume_unmounted",
 		    G_CALLBACK (volume_changed), NULL);
 	
-	GnomeVFSResult result =gnome_vfs_monitor_add(&handle_home,g_get_home_dir(),
+	result = gnome_vfs_monitor_add(&handle_home,g_get_home_dir(),
 										GNOME_VFS_MONITOR_DIRECTORY,
 										contents_changed_cb,NULL);
 	
@@ -562,7 +632,7 @@ baobab_init(void)
 
 }
 
-void
+static void
 baobab_close(void)
 {
 		g_free(baobab.label_scan);
@@ -570,6 +640,7 @@ baobab_close(void)
 		g_string_free(baobab.last_scan_command,TRUE);
 		g_string_free(bbSearchOpt.filename,TRUE);
 		g_string_free(bbSearchOpt.dir,TRUE);
+		g_free(baobab.glade_path);
 	
 	if (handle_mtab) {
 		gnome_vfs_monitor_cancel(handle_mtab);
@@ -577,60 +648,52 @@ baobab_close(void)
 	if (handle_home) {
 		gnome_vfs_monitor_cancel(handle_home);
 	}
-	
+
+	g_free(baobab.selected_path);
+
 	g_slist_foreach(baobab.bbExcludedDirs,(GFunc)g_free,NULL);
 	g_slist_free(baobab.bbExcludedDirs);
+	glibtop_close();
 	gnome_vfs_shutdown();
 }
-////////////////////////////////main///////////////////////////////
+
+
+
 int
 main (int argc, char *argv[])
 {
- 
+	GdkPixbuf * window_icon;
 
-#ifdef ENABLE_NLS
-  setlocale(LC_MESSAGES, "");
-  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
-//  bindtextdomain(GETTEXT_PACKAGE,"/usr/share/locale");
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-  textdomain (GETTEXT_PACKAGE);
-#endif
+	bindtextdomain (GETTEXT_PACKAGE, BAOBAB_LOCALE_DIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
 
-
-  	gtk_set_locale ();
-  	//gtk_init (&argc, &argv);
 	gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE,
-	                            argc, argv,GNOME_PARAM_NONE);
-	g_type_init();
-	
-	
-	/* remember to initialize GnomeVFS! */
-  	if (!gnome_vfs_init ()) {
-    	printf ("Could not initialize GnomeVFS\n");
-    	return 1;
-  	}
-	
-	add_pixmap_directory(PIX_DIR);
-	
-	//initialize variables
+			   argc, argv,
+			   GNOME_PARAM_APP_DATADIR, BAOBAB_DATA_DIR, NULL);
+
+	g_assert(gnome_vfs_init());
+	gnome_authentication_manager_init ();
+	glibtop_init();
+
 	baobab_init();
 	
-	g_fs=getFileSystem();
+	baobab_get_filesystem(&g_fs);
 	set_label_scan(&g_fs);
 	
-  	baobab.window = create_window ();
+  	baobab.window = glade_xml_get_widget(baobab.main_xml,"baobab_window");
 	gtk_window_set_position(GTK_WINDOW(baobab.window),GTK_WIN_POS_CENTER);
-  	GdkPixbuf * window_icon = create_pixbuf("baobab.png");
+	window_icon = baobab_get_icon();
   	gtk_window_set_icon(GTK_WINDOW(baobab.window),window_icon);	
-  	gtk_window_set_default_size(GTK_WINDOW(baobab.window),-1,500);
+	gtk_window_set_default_icon(window_icon);
 	
-	//set global pixbufs
-	baobab.yellow_bar=create_pixbuf("yellow.png");
-	baobab.red_bar=create_pixbuf("red.png");
-	baobab.green_bar=create_pixbuf("green.png");
+	/* set global pixbufs */
+	baobab.yellow_bar=baobab_load_pixbuf("yellow.png");
+	baobab.red_bar=baobab_load_pixbuf("red.png");
+	baobab.green_bar=baobab_load_pixbuf("green.png");
 	
-	baobab.tree_view = create_treeview();
-	baobab.tree_search = create_tree_search();
+	baobab.tree_view = create_directorytreeview();
+	baobab.tree_search = create_tree_filesearch();
 	set_label_search(0,0);
 	
 	switch_view(VIEW_SEARCH);
@@ -638,29 +701,26 @@ main (int argc, char *argv[])
 	show_label(VIEW_TREE);
 	change_icon("done.png");
 		
-	//set allocated space checkbox
-	gtk_toggle_button_set_active((GtkToggleButton*)lookup_widget(baobab.window,"ck_allocated"),
+	/* set allocated space checkbox */
+	gtk_toggle_button_set_active((GtkToggleButton*)glade_xml_get_widget(baobab.main_xml,"ck_allocated"),
 								baobab.show_allocated);
 								
 	gtk_widget_show (baobab.window);
 
 	first_row();
 	set_statusbar(_("Ready"));
-	
-	
-	if (install_naut()){
-		message(_("A new script has been generated to open Baobab from Nautilus. Just right-click in Nautilus->Script to access Baobab. "),baobab.window);
-	}
-	
-	//commandline
+
+	/* commandline */ 
 	if (argc > 1){
-		start_proc_on_dir(argv[1]);
+		gchar *uri_shell;
+		uri_shell = gnome_vfs_make_uri_from_shell_arg(argv[1]);
+		start_proc_on_dir(uri_shell);
+		g_free(uri_shell);
 		}
 		
 			
  	gtk_main ();
 	
-	//free variables;
 	baobab_close();
 
 return 0;
