@@ -5,12 +5,12 @@
  *  Copyright  2005  Fabio Marzocca
  *  Email
  ****************************************************************************/
-#ifndef __TV_H__
-#define __TV_H__
+#ifndef BAOBAB_TV_H
+#define BAOBAB_TV_H
 
 #include <gtk/gtk.h>
 
-// tree model columns (_H_ are hidden)
+/* tree model columns (_H_ are hidden) */
 enum
 {
 	COL_DIR_NAME,
@@ -29,7 +29,7 @@ enum
 	
 };
 
-// search model columns
+/* search model columns */
 enum
 {
   COL0_ICON,
@@ -45,8 +45,8 @@ enum
 
 
 
-GtkWidget* create_treeview(void);
+GtkWidget* create_directorytreeview(void);
 
-GtkWidget* create_tree_search (void);
+GtkWidget* create_tree_filesearch (void);
 
 #endif
